package serve

import (
	"context"
	"flag"
	"time"

	"git.tcp.direct/hgc/mailhole/config"

	"github.com/boltdb/bolt"
	"github.com/emersion/go-smtp"
	"github.com/google/subcommands"
	"github.com/lrstanley/girc"
	"github.com/rs/zerolog/log"
)

type ServeCommand struct {
}

func (*ServeCommand) Name() string     { return "serve" }
func (*ServeCommand) Synopsis() string { return "Serve mailhole services" }
func (*ServeCommand) Usage() string {
	return `serve:
  Serves SMTP server & web UI as per config`
}
func (p *ServeCommand) SetFlags(f *flag.FlagSet) {}

var t *config.MailholeConfig
var ircClients map[string]*girc.Client
var attachmentDb *bolt.DB

func (p *ServeCommand) Execute(_ context.Context, f *flag.FlagSet, _ ...interface{}) subcommands.ExitStatus {
	// BoltDB is for storing mail attachments to disk to keep memory low
	var err error
	if attachmentDb, err = bolt.Open("attachment.db", 0600, nil); err != nil {
		log.Fatal().Stack().Err(err).Msg("Failed to open attachment db")
	}

    // We don't care about the error - if it fails it will fatal
	attachmentDb.Update(func(tx *bolt.Tx) error {
		if _, err := tx.CreateBucket([]byte("attachments")); err != nil {
            return err
		}
		if _, err := tx.CreateBucket([]byte("attachments_types")); err != nil {
            return err
		}
        return nil
	})

	receivedEmails = map[string][]*ReceivedEmail{}
	ircClients = map[string]*girc.Client{}
	t = config.LoadConfig()
	for k, v := range t.IRC {
		log.Debug().Str("name", k).Msg("Constructing girc config")
		ircClients[k] = girc.New(girc.Config{
			Server:     v.Server,
			ServerPass: v.ServerPass,
			Port:       v.Port,
			Nick:       v.Nick,
			User:       v.User,
			Name:       v.Name,
			SSL:        v.SSL,
			AllowFlood: v.AllowFlood,
		})
	}

	for k, v := range ircClients {
		log.Info().Str("name", k).Msg("Connecting IRC client")
		go v.Connect()

		v.Handlers.Add(girc.CONNECTED, func(c *girc.Client, e girc.Event) {
			joinChan := t.IRC[k].Channel
			log.Info().Str("name", k).Str("chan", joinChan).Msg("Connected, joining channel")
			c.Cmd.Join(joinChan)
		})
	}

	log.Info().Msg("Main thread now serving")
    
    // Attachments: Serve plain
    go ServeAttachments()

	// SMTP: Initialization
	be := &Backend{}

	// SMTP: Serve plain
	if t.SMTP.PlainEnabled {
		s := smtp.NewServer(be)
		s.Addr = t.SMTP.PlainBind
		s.Domain = t.SMTP.Domain
		s.ReadTimeout = 10 * time.Second
		s.WriteTimeout = 10 * time.Second
		s.MaxMessageBytes = 1024 * 1024
		s.MaxRecipients = 50
		s.AllowInsecureAuth = true

		log.Info().Str("PlainBind", s.Addr).Msg("Starting SMTP server")
		go func() {
			if err := s.ListenAndServe(); err != nil {
				log.Error().Stack().Err(err).Msg("Failed to serve plain SMTP server")
			}
		}()
	}

	// SMTP: Serve TLS

	//SMTP serve goes here
	for {
		time.Sleep(5 * time.Second)
	}

	return subcommands.ExitSuccess
}
