package serve

import (
	"bytes"
	"errors"
	"io"
	"io/ioutil"
	"strings"

	"github.com/boltdb/bolt"
	"github.com/emersion/go-smtp"
	"github.com/jhillyerd/enmime"
	"github.com/lrstanley/girc"
	"github.com/rs/zerolog/log"

	"git.tcp.direct/hgc/mailhole/utils"
)

// The Backend implements SMTP server methods.
type Backend struct{}

func (bkd *Backend) NewSession(_ *smtp.Conn) (smtp.Session, error) {
	return &Session{}, nil
}

// A Session is returned after EHLO.
type Session struct {
	// Store the From & To addresses when they come through in this session,
	// so they can be queued up with Data
	From string
	To   []string
}

func (s *Session) AuthPlain(username, password string) error {
	return errors.New("Server does not support authentication")
}

type ReceivedEmail struct {
	From           string
	Data           []byte
	HTMLKey        []byte
	TextKey        []byte
	AttachmentKeys [][]byte
}

var receivedEmails map[string][]*ReceivedEmail

func (s *Session) Mail(from string, opts *smtp.MailOptions) error {
	s.From = from
	return nil
}

func (s *Session) Rcpt(to string) error {
	s.To = append(s.To, to)
	return nil
}

func (s *Session) Data(r io.Reader) error {
	if b, err := ioutil.ReadAll(r); err != nil {
		return err
	} else {
		q := &ReceivedEmail{
			From:           s.From,
			Data:           b,
			AttachmentKeys: [][]byte{},
		}

		// Parse email
		parsedMsg, err := enmime.ReadEnvelope(bytes.NewReader(b))
		if err != nil {
			log.Error().Stack().Err(err).Msg("Error parsing incoming message")
			return err
		}

		subj := parsedMsg.GetHeader("Subject")

		log.Info().Str("subject", subj).Str("from", s.From).Strs("to", s.To).Msg("Received")

		// Parse attachments & put in attachment bucket
		attachmentDb.Update(func(tx *bolt.Tx) error {
			ba := tx.Bucket([]byte("attachments"))
			bt := tx.Bucket([]byte("attachments_types"))

			if len(parsedMsg.Text) > 0 {
				q.TextKey = utils.RandByteRunes(16)
				ba.Put(q.TextKey, []byte(parsedMsg.Text))
				bt.Put(q.TextKey, []byte("text/plain"))
				log.Info().Str("type", "text").Bytes("key", q.TextKey).Msg("Stored an attachment")
			}

			if len(parsedMsg.HTML) > 0 {
				q.HTMLKey = utils.RandByteRunes(16)
				ba.Put(q.HTMLKey, []byte(parsedMsg.HTML))
				bt.Put(q.TextKey, []byte("text/html"))
				log.Info().Str("type", "HTML").Bytes("key", q.HTMLKey).Msg("Stored an attachment")
			}

			for _, v := range parsedMsg.Attachments {
				key := utils.RandByteRunes(16)
				ba.Put(key, v.Content)
				bt.Put(key, []byte(v.ContentType))
				q.AttachmentKeys = append(q.AttachmentKeys, key)
				log.Info().Str("type", v.ContentType).Bytes("key", key).Msg("Stored an attachment")
			}
			return err
		})

		attachmentDb.View(func(tx *bolt.Tx) error {
			bt := tx.Bucket([]byte("attachments_types"))

			for _, addr := range s.To {
				if !strings.Contains(addr, "@") {
					continue
				}

				// Add to the list of received emails for display in the web ui
				receivedEmails[addr] = append(receivedEmails[addr], q)

				// Get mailDomain
				mailDomain := strings.Split(addr, "@")[1]

				for _, x := range t.Domains {
					if strings.ToLower(mailDomain) == strings.ToLower(x.Domain) {

						msgs := formatNewMail(s.From, addr, subj, q.HTMLKey, q.TextKey, q.AttachmentKeys, bt)

						for _, msg := range msgs {
							ircClients[x.IRC].Cmd.Message(
								t.IRC[x.IRC].Channel,
								msg,
							)
						}
					}
				}
			}
			return nil
		})
	}
	return nil
}

func formatNewMail(from string, to string, subject string, htmlKey []byte, textKey []byte, attachments [][]byte, bt *bolt.Bucket) []string {
	msgs := []string{}
	msgs = append(msgs, "+ "+girc.Fmt("{red}")+from+girc.Fmt("{c}")+" -> "+girc.Fmt("{green}")+to+girc.Fmt("{c}")+" "+subject)

	if string(htmlKey) != "" {
		msgs = append(msgs, "| HTML: "+t.WebUIBase+`/`+string(htmlKey))
	}

	if string(textKey) != "" {
		msgs = append(msgs, "| Text: "+t.WebUIBase+`/`+string(textKey))
	}

	for _, a := range attachments {
		msgs = append(msgs, "+ Attachment: "+t.WebUIBase+`/`+string(a)+` (`+string(bt.Get(a))+`)`)
	}

	msgs = append(msgs, "+---")

	return msgs
}

func (s *Session) Reset() {}

func (s *Session) Logout() error {
	return nil
}
